local function getOption(key)
	return SpellReminder.db.profile[key]
end
local function setOption(key, value)
	SpellReminder.db.profile[key] = value
	SpellReminder:RefreshAllBars()
end

local defaultSpells = {
	type = "group",
	order = 1,
	name = "Spell Setup",
	desc = "Setup spell reminders.",
	args = {
		inactive = {
			type = "group",
			order = 1,
			name = "Inactive",
			desc = "Inactive Spells",
			args = {},
			hidden = true,
		},
		inactiveSplit = { type = "header", order = 2, hidden = true, },
		spacer = { type = "header", order = 100 },
		--[[addNew = {
			type = "text",
			order = 101,
			name = "Add New Spell",
			desc = "Add a new spell to the watch list.",
			usage = "<name>",
			get = function () return "" end,
			set = function (v) SpellReminder:InstallWatch(v) SpellReminder:RefreshSpellOptions() end,
		},]]
		disableDetection = {
			type = "toggle",
			order = 103,
			name = "Disable Detection",
			desc = "Disables the detection of new spells",
			get = getOption,
			set = setOption,
			passValue = "disableDetection",
		},
		spacer2 = { type = "header", order = 105 },
		defaultcolor = {
			type = "color",
			order = 107,
			name = "Default Bar Color",
			desc = "Set the default color of the reminder bars.",
			get = function () local c = getOption("defaultBarColor") return c.r,c.g,c.b,c.a end,
			set = function (r, g, b, a) setOption("defaultBarColor", {["r"]=r, ["g"]=g, ["b"]=b, ["a"]=1}) SpellReminder:RefreshSpellOptions() end,					
		},
		spacer2 = { type = "header", order = 108 },
		ClearAll = {
			type = "execute",
			order = 110,
			name = "Clear All Spells",
			desc = "Will force SpellReminder to forget all options for ALL spells, including detected spells and pet spells.",
			func = function () SpellReminder:ClearSpells(true) end,
		},
		addAll = {
			type = "execute",
			order = 115,
			name = "Scan Spellbook",
			desc = "Attempts to scan your spellbook and watch all spells.",
			func = "InstallAllSpells",
		},
	},
}

function SpellReminder:InitOptions()
	return {
		type = "group",
		name = "SpellReminder",
		desc = "Options for SpellReminder.",
		handler = SpellReminder,
		args = {
			spells = defaultSpells,
			spacer2 = { type = "header", order = 2 },
			barSetup = {
				type = "group",
				order = 3,
				name = "Timer Bars",
				desc = "Setup bar animation options and scaling.",
				args = {
				--[[	startScale = {				
						type = "range",
						order = 1,
						name = "Start Scale",
						desc = "Default Bar Scale.",
						min = 0.2,
						max = 2.0,
						step = 0.1,
						get = getOption,
						set = setOption,
						passValue = "startScale",
					},
					stopScale = {
						type = "range",
						order = 3,
						name = "Emphasize Scale",
						desc = "Emphasized Bar Scale.",
						disabled = function () return getOption("disableEmphasis") == true end,
						min = 0.2,
						max = 3.0,
						step = 0.1,
						get = getOption,
						set = setOption,
						passValue = "stopScale",
					},]]
			--[[		ttg = {
						type = "range",
						order = 4,
						name = "Tween Time",
						desc = "Total time of tween animation.",
						visible = false,
						disabled = function () return getOption("disableEmphasis") == true end,
						min = 0.1,
						max = 3.0,
						step = 0.1,
						get = getOption,
						set = setOption,
						passValue = "ttg",
					},
					disableEmphasis = {
						type = "toggle",
						order = 2,
						name = "Disable Emphasis",
						desc = "Disables the emphasise effect",
						get = getOption,
						set = setOption,
						passValue = "disableEmphasis",
					},
					spacer2 = { type = "header", order = 5 },]]
					barGroups = {
						type = "group",
						order = 1,
						name = "Bar Groups",
						desc = "setup groups",
						args = {},
					},
					filters = {
						type = 'group',
						order = 5,
						name = "Filter",
						desc = "Filter different spell types.",
						args = {
							self = {
								type = "text",
								order = 10,
								name = "Self",
								desc = "Which group should self casts be sent.",
								usage = "<select>",
								validate = SpellReminder.BarGroupList,
								get = function () return string.format("%d",getOption("targetGroup_self")) end,
								set = function (v) setOption("targetGroup_self",tonumber(v)) end,
							},
							buffs = {
								type = "text",
								order = 10,
								name = "Buffs",
								desc = "Which group should buffs be sent.",
								usage = "<select>",
								validate = SpellReminder.BarGroupList,
								get = function () return string.format("%d",getOption("targetGroup_buff")) end,
								set = function (v) setOption("targetGroup_buff",tonumber(v)) end,
							},
							debuffs = {
								type = "text",
								order = 10,
								name = "Debuffs",
								desc = "Which group should debuffs be sent.",
								usage = "<select>",
								validate = SpellReminder.BarGroupList,
								get = function () return string.format("%d",getOption("targetGroup_debuff")) end,
								set = function (v) setOption("targetGroup_debuff",tonumber(v)) end,
							},
						},
					},
					spacer1 = { type = "header", order = 10 },
					lockBars = {
						type = "toggle",
						order = 15,
						name = "Lock timer bars",
						desc = "Lock the position of the timer bars.",
						get = function () return getOption("lockbars") end,
						set = function (v)
							setOption("lockbars",v)
							-- show/hide the anchors
							SpellReminder:ToggleAnchors()
						end
					},
					resetBars = {
						type = "execute",
						order = 20,
						name = "Reset Bars",
						desc = "Reset the posion of the bars.",
						func = "ResetAnchors",
					},
					testBars = {
						type = "execute",
						order = 25,
						name = "Test Bars",
						desc = "Create sample reminder bars to test your layout.",
						func = "CreateTestBars",
					},
					spacer10 = { type = "header", order = 30 },
					formattingSub = {
						type = 'group',
						order = 35,
						name = "Bar Formatting",
						desc = "Various formating options for the bars.",
						args = {
							textFormat = {
								type = 'text',
								name = "Display Text",
								desc = "Set the format of the text that appears on the bars.  Possible values are: %n (spell name) %r (spell rank) %t (target) %a (applications)",
								usage = "<format>",
								order = 11,
								get = getOption,
								set = setOption,
								passValue = 'textFormat',
							},
							texture = {
								type = 'text',
								name = "Texture",
								desc = "Set the Cast Bar Texture",
								validate = SpellReminder.sm:List('statusbar'),
								order = 12,
								get = getOption,
								set = setOption,
								passValue = 'texture',
							},
							spacer3 = { type = "header", order = 100 },
							barwidth = {
								type = 'range',
								name = "Bar Width",
								desc = "Set the Cast Bar Width",
								order = 110,
								min = 50,
								max = 500,
								step = 10,
								get = getOption,
								set = setOption,
								passValue = 'barwidth',
							},
							barheight = {
								type = 'range',
								name = "Bar Height",
								desc = "Set the Cast Bar Height",
								order = 115,
								min = 5,
								max = 40,
								step = 1,
								get = getOption,
								set = setOption,
								passValue = 'barheight',
							},
							spacer4 = { type = "header", order = 120 },
							barfont = {
								type = 'text',
								name = "Font",
								desc = "Set the Cast Bar Font",
								validate = SpellReminder.sm:List('font'),
								order = 125,
								get = getOption,
								set = setOption,
								passValue = 'barfont',
							},
							barfontsize = {
								type = 'range',
								name = "Font Size",
								desc = "Set the Cast Bar Font Size",
								order = 130,
								min = 1,
								max = 30,
								step = 1,
								get = getOption,
								set = setOption,
								passValue = 'barfontsize',
							},
						},
					},
				},
			},
			debugmode = {
				type = "toggle",
				order = 5,
				name = "Debug Mode",
				desc = "Show Debug Messages.",
				get = function () return getOption("debugmode") end,
				set = function (v) setOption("debugmode",v) end,
			},
			spacer4 = { type = "header", order = 6 },
			textWarnings = {
				type = 'text',
				name = "Text Warnings",
				desc = "Select which Text Warnings to display",
				validate = { "Both" , "Expiring Only" , "Expired Only" },
				order = 7,
				get = getOption,
				set = setOption,
				passValue = 'textWarnings',
			},
			audioWarnings = {
				type = 'text',
				name = 'Audiable Warnings',
				desc = "Play a sound when a spell is about to expire.",
				validate = SpellReminder.sm:List('sound'),
				order = 8,
				get = getOption,
				set = function(_,value) PlaySoundFile(SpellReminder.sm:Fetch('sound',value)) setOption("audioWarnings",value) end,
				passValue = 'audioWarnings',
			},
			spacer5 = { type = "header", order = 10 },
		},
	}
end

function SpellReminder:RefreshGroupOptions()
	self.options.args.barSetup.args.barGroups.args = { -- setup
			-- add/delete 
		addGroup = {
			type = "text",
			order = 1,
			name = "Add New",
			desc = "name of the anchor",
			usage = "<name>",
			--disabled = (# self.anchors) > (# SpellReminder.barTypes), 
			get = function () return "" end,
			set = function (v) self:AddNewAnchor({name=v}) end
		},
		spacer = { type = "header", order = 2 },
	}
	table.foreach(self.anchors, function (i,anchor)
		self.options.args.barSetup.args.barGroups.args[i] = self:GetGroupOptions(i)
	end)
end

SpellReminder.BarGroupList = {["-1"]="None"}
function SpellReminder:RefreshBarGroupList()
	local list = { ["-1"]="None"}
	for k,n in ipairs(self.anchors) do
		list[tostring(k)] = n.name
	end
	
	SpellReminder.BarGroupList = list
	self.options.args.barSetup.args.filters.args.self.validate = list
	self.options.args.barSetup.args.filters.args.buffs.validate = list
	self.options.args.barSetup.args.filters.args.debuffs.validate = list
end

function SpellReminder:GetGroupOptions(id)
	local anchor = self.anchors[id]
	return {
			type = "group",
			name = "Bar: "..anchor.name,
			desc = "setup anchor",
			order = 50+id,
			args = {
				anchorName = {
					type = "text",
					order = 1,
					name = "Anchor Name",
					desc = "name of the anchor",
					usage = "<name>",
					disabled = anchor.cantEdit,
					get = function () return self.db.char.bargroups[id].name end,
					set = function (v) self.db.char.bargroups[id].name = v self:CreateAnchors()  end,
				},
				growup = {
					type = "toggle",
					order = 1,
					name = "Grow Upward",
					desc = "Bars stack ontop of each other instead of underneath each other.",
					get = function () return self.db.char.bargroups[id].growup or false end,
					set = function (v) self.db.char.bargroups[id].growup = v self.candybar:SetCandyBarGroupGrowth("ReminderCandy_"..id, v) end,
				},
				emphasise = {
					type = "toggle",
					order = 1,
					name = "Emphasise",
					desc = "Bars from this group will emphasise.",
					get = function () return self.db.char.bargroups[id].emphasise or false end,
					set = function (v) self.db.char.bargroups[id].emphasise = v end,
				},
				scale = {				
					type = "range",
					order = 1,
					name = "Bar Scale",
					desc = "Default Bar Scale.",
					min = 0.2,
					max = 3.0,
					step = 0.1,
					get = function () return self.db.char.bargroups[id].scale or 1 end,
					set = function (v) self.db.char.bargroups[id].scale = v SpellReminder:RefreshAllBars() end,
				},
				deleteGroup = {
					type = "execute",
					name = "Delete",
					desc = "Delete this group",
					order = -1,
					disabled = anchor.cantEdit,
					func = function () self.anchors[id].frame:Hide() self.anchors[id] = nil self.db.char.bargroups[id] = nil self:CreateAnchors() end
				},
			},
		}
end

function SpellReminder:RefreshSpellOptions()
	local tmpTbl={}
	foreach(self.db.char.spellSetup,function (i,v) tinsert(tmpTbl,i) end)
	sort(tmpTbl)
	self.options.args.spells = tcopy(defaultSpells)
	foreachi(tmpTbl,function(i,spellName)
		local info = self.db.char.spellSetup[spellName]
		if info == nil or info.d == nil or info.d == 0 then return end
		local c = self.db.char.spellSetup[spellName].c or getOption("defaultBarColor")
		local colorString = "|cFF"..string.format("%2x",c.r*255)..string.format("%2x",c.g*255)..string.format("%2x",c.b*255)
--		if durationBuff == nil and not info.detected then return end -- this can happen if the spell isnt available on this character, ie, it has been untrained
		local spellGroup = {
			type = "group",
			order = i+2,
			icon = info.icon,
			name = colorString..spellName,  -- |cAARRGGBB
			desc = "Setup reminder for "..spellName,
			args = {
				active = {
					type = "toggle",
					order = 1,
					name = "Active",
					desc = "Track this spell?",
					get = function () return self.db.char.spellSetup[spellName].active end,
					set = function (v) self.db.char.spellSetup[spellName].active = v SpellReminder:RefreshSpellOptions() end,
				},
				selfOnly = {
					type = "toggle",
					order = 1,
					name = "Self Only",
					desc = "Only show this spell if its cast on yourself. (Useful for AoE and Channeled spells)",
					disabled = function () return self.db.char.spellSetup[spellName].active == false end,
					get = function () return self.db.char.spellSetup[spellName].selfOnly end,
					set = function (v) self.db.char.spellSetup[spellName].selfOnly = v end,
				},
--				sharedName = {
--					type = "text",
--					order = 2,
--					name = "Shared Name",
--					desc = "A common name across unique spells, EG: only one warrior shout can be active at once.",
--					disabled = function () return self.db.char.spellSetup[spellName].active == false end,
--					get = function () return self.db.char.spellSetup[spellName].s end,
--					set = function (v) self.db.char.spellSetup[spellName].s = v end,
--					usage = "<name>",
--				},
				displayName = {
					type = "text",
					order = 2,
					name = "Display Name",
					desc = "Spell name text to display on the bar. For example an abbreviation.",
					disabled = function () return self.db.char.spellSetup[spellName].active == false end,
					get = function () return self.db.char.spellSetup[spellName].displayName end,
					set = function (v) self.db.char.spellSetup[spellName].displayName = v SpellReminder:RefreshAllBars() end,
					usage = "<name>",
				},
				reminder = {
					type = "range",
					order = 3,
					name = "Reminder",
					desc = "Reminder occurs when the spell duration remaining reaches this value.",
					min = 0,
					max = info.d,
					step = 1,
					disabled = function () return self.db.char.spellSetup[spellName].active == false end,
					get = function () return self.db.char.spellSetup[spellName].r end,
					set = function (v) self.db.char.spellSetup[spellName].r = v end,
				},
				color = {
					type = "color",
					order = 5,
					name = "Bar Color",
					desc = "Set the color of the reminder bar.",
					disabled = function () return self.db.char.spellSetup[spellName].active == false end,
					get = function () local c = self.db.char.spellSetup[spellName].c or getOption("defaultBarColor") return c.r,c.g,c.b,c.a end,
					set = function (r, g, b, a) self.db.char.spellSetup[spellName].c = {["r"]=r, ["g"]=g, ["b"]=b, ["a"]=1} self:RefreshSpellOptions() SpellReminder:RefreshAllBars() end,					
				},
				sep3 = { type = "header", order = 10 },
				persistent = {
					type = "toggle",
					order = 12,
					name = "Persistent",
					desc = "If selected, this bar will stay active regardless of death or dispel.",
					get = function () return self.db.char.spellSetup[spellName].persistent == true end,
					set = function (v) self.db.char.spellSetup[spellName].persistent = v end,
				},
				fadeTime = {
					type = "range",
					order = 13,
					name = "Fade Time",
					desc = "Time until the bar disapears after it reaches zero. -1 to always show.",
					min = -1,
					max = 10,
					step = 1,
					get = function () return self.db.char.spellSetup[spellName].fade or 0 end,
					set = function (v) self.db.char.spellSetup[spellName].fade = v end,
				},
				sep = { type = "header", order = 15 },
				customSound = {
					type = "toggle",
					order = 20,
					name = "Allow Custom Sound",
					desc = "Allow selection of a custom sound file for this spell.",
					get = function () return self.db.char.spellSetup[spellName].allowCustomSound == true end,
					set = function (v) self.db.char.spellSetup[spellName].allowCustomSound = v end,
				},
				soundFile = {
					type = 'text',
					order = 21,
					name = 'Custom Sound',
					desc = "Play a sound when a spell is about to expire.",
					validate = SpellReminder.sm:List('sound'),
					disabled = function () return self.db.char.spellSetup[spellName].allowCustomSound ~= true end,
					get = function(value) return self.db.char.spellSetup[spellName].customSound or "" end,
					set = function(value) PlaySoundFile(SpellReminder.sm:Fetch('sound',value)) self.db.char.spellSetup[spellName].customSound = value end,
				},
				sep2 = { type = "header", order = 25 },
				customGroup = {
					type = "toggle",
					order = 30,
					name = "Allow Custom Group",
					desc = "Allow selection of a custom group for this spells timer bar to appear.",
					get = function () return self.db.char.spellSetup[spellName].allowCustomGroup == true end,
					set = function (v) self.db.char.spellSetup[spellName].allowCustomGroup = v end,
				},
				barGroup = {
					type = "text",
					order = 31,
					name = "Custom Group",
					desc = "Which group should this spell be sent to.",
					usage = "<select>",
					validate = SpellReminder.BarGroupList,
					disabled = function () return self.db.char.spellSetup[spellName].allowCustomGroup ~= true end,
					get = function () return string.format("%d",self.db.char.spellSetup[spellName].customGroup or 1) end,
					set = function (v) self.db.char.spellSetup[spellName].customGroup = tonumber(v) end,
				},
			}
		}
		if not self.db.char.spellSetup[spellName].active then
			self.options.args.spells.args["spell_"..string.gsub(spellName," ","_")] = nil
			self.options.args.spells.args.inactive.args["spell_"..string.gsub(spellName," ","_")] = spellGroup
			if self.options.args.spells.args.inactive.hidden then
				self.options.args.spells.args.inactive.hidden = false
				self.options.args.spells.args.inactiveSplit.hidden = false
			end
		else
			self.options.args.spells.args.inactive.args["spell_"..string.gsub(spellName," ","_")] = nil
			self.options.args.spells.args["spell_"..string.gsub(spellName," ","_")] = spellGroup
		end
	end)
end

function tcopy(from,to)   -- "to" must be a table (possibly empty)
	if type(from)~="table" then return nil end
	if to == nil then to = {} end
	for k,v in pairs(from) do
		if(type(v)=="table") then
			to[k] = tcopy(v)
		else
			to[k] = v
		end
	end
	
	return to
end
 